/***************************************************************************
 *   Program MSX Maracana classic score - gif2dump                        *
 *                                                                         *
 *   Copyright (C) 2020 by Marcelo Teixeira Silveira, D.Sc.                *
 *   MSX Sketch Tools: http://marmsx.msxall.com                            *
 *   Marcelo Teixeira Silveira is Computer Engineer,                       *
 *   graduated at Universidade do Estado do Rio de Janeiro (UERJ)          *
 *   Contact: flamar98@hotmail.com                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.awt.Graphics2D;
import java.awt.Color;

public class gif2dump {

	private static BufferedImage img;
	private static int gab_width=516, gab_height=151;

	// Abre e verifica se imagem é compatível com o gabarito
	private static void loadImage(String filename) {
		// Abre gabarito 
		try {
			img = ImageIO.read(new File(filename));
		}
		catch(IOException e) {
			System.out.println("Erro ao abrir o arquivo "+filename+".");
			System.exit(0);
		}

		// Verifica se é válido
		if (img.getWidth() != gab_width || img.getHeight() != gab_height) {
			System.out.println("Gabarito inválido. Imagem deve ter "+gab_width+" x "+gab_height+" pixels.");
			System.exit(0);
		}
	}

	// Converte para dump de screen 0 do MSX 1 - lembre-se: blocos de 6x8
	private static void convertToDump(String filename) {
		int step_x=103, step_y=25;
		byte [] msx_dump = new byte[12288];
		int p=7; // Dump pointer

		// Add header
		msx_dump[0]=(byte) 0xFE; msx_dump[1]=(byte) 0x00; msx_dump[2]=(byte) 0x10;
		msx_dump[3]=(byte) 0xFF; msx_dump[4]=(byte) 0x3F; msx_dump[5]=(byte) 0x00; msx_dump[6]=(byte) 0x10;

		// Estrutura mais externa: janelas de 6 x 5
		for (int i=1; i<img.getHeight(); i+=step_y) {
			for (int j=1; j<img.getWidth(); j+=step_x) {

				// Estrutura mais interna - bloco de 102 x 24 pixels
				for (int y=i; y<i+step_y-1; y+=8) {
					for (int x=j; x<j+step_x-1; x+=6) {

						// Percorre blocos de 6x8
						for (int ln=y; ln<y+8; ln++)
							msx_dump[p++] = line2byte(x,ln);
					}
				}
			}
		}

		// Salva dados
		try {
			OutputStream os = new FileOutputStream(filename);
			os.write(msx_dump);
		}
		catch(IOException e) {
			System.out.println("Erro ao tentar salvar o dump de imagem.");
			System.exit(0);
		}
	}

	// Converte de linha para byte
	private static byte line2byte(int x, int y) {
		byte bt=0;

		for (int r=0; r<6; r++) {
			if ((img.getRGB(x+r,y) & 0xFF) > 128)
				bt += 1 << (7-r);
		}

		return bt;
	}

	// Cria um novo gabarito e salva como "gabarito.gif"
	private static void createModel() {
		int width=516, height=151;
		BufferedImage img_gab = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = img_gab.createGraphics();

		// Preenche de preto
		g2d.setColor(new Color(0,0,0));
		g2d.fillRect(0,0,width,height);

		// Cria grid
		g2d.setColor(new Color(128,128,128));
		for (int y=0; y<height; y+=25)
			g2d.drawLine(0, y, width-1, y);
		for (int x=0; x<width; x+=103)
			g2d.drawLine(x, 0, x, height-1);

		// Salva
		try {
			ImageIO.write(img_gab, "gif", new File("gabarito.gif"));
		}
		catch(IOException e) {
			System.out.println("Erro ao tentar salvar o gabarito.");
			System.exit(0);
		}
	}

	private static void help() {
		System.out.println("-= Placar clássico do Maracanã - gif2dump - MarMSX 2020 =-");
		System.out.println("Converte imagens do placar do Maracanã para dump de memória de vídeo do MSX 1.");
		System.out.println("Por favor, utilize o gabarito.gif para criar as imagens em preto e branco da área gráfica do placar.");
		System.out.println("Então converta a imagem criada assim:");
		System.out.println(" java gif2dump imagem_do_gabarito.gif nome_dump.grp");
		System.out.println("Exemplo:");
		System.out.println(" java gif2dump meu_placar.gif meu_placar.grp");
		System.out.println("Para gerar um novo arquivo 'gabarito.gif', use:");
		System.out.println(" java gif2dump -gabarito");
	}

	public static void main(String args []) {
		if (args.length == 1) {
			if (args[0].equals("-gabarito")) {
				createModel();
				System.out.println("Gabarito salvo como 'gabarito.gif'.");
				System.exit(0);
			}
		}

		if (args.length != 2) {
			help();
			System.exit(0);
		} 
			
		loadImage(args[0]);
		convertToDump(args[1]);
		System.out.println("Arquivo "+args[1]+" salvo com sucesso.");
	}
}
